import 'dart:async';
import 'package:_private_chat/firestore_resouce/instances.dart';
import 'package:_private_chat/options.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:get/get.dart';
import 'package:_private_chat/chat_detail/chat_detail_screen.dart';
import 'package:_private_chat/chat_detail/chat_helper.dart';
import 'package:_private_chat/utils/utils.dart';
import 'package:_private_chat/constant.dart';

import '../../connection_status/bloc/connection_status_bloc.dart';

part 'chat_list_event.dart';
part 'chat_list_state.dart';

String chatGetFullNameUser(data) =>
    data == null ? '' : data['displayName'] ?? data['username'] ?? '';

String chatGetThumbnailUser(data) => data == null ? '' : data['thumbnail'];

String chatGetUsernameUser(data) => data == null ? '' : data['username'];

class ChatListConversation {
  int timestamp;
  ChatConversationType conversationType;
  String conversationId;
  Map data = {};
  List userIds = [];
  int unread;
  Map<String, dynamic> lastMessage = {};
  bool isEmptyMessages;
  bool isBot;

  ChatListConversation({
    this.timestamp = 0,
    this.isBot = false,
    this.conversationId = '',
    this.conversationType = ChatConversationType.solo,
    this.unread = 0,
    this.isEmptyMessages = false,
  });
}

class ChatListBloc extends Bloc<ChatListEvent, ChatListState> {
  StreamSubscription? _converstationsSubscription;
  final List<StreamSubscription> _msgsSubscription = [];

  ChatListBloc() : super(ChatListState(conversations: [], usersByKeys: {})) {
    on<ChatListInitEvent>(_onInitial);
    on<ChatListReadItemEvent>(_onReadItemEvent);
    on<ChatListDisposeEvent>(_onDisposeEvent);
    on<ChatListRemoveEvent>(onRemoveEvent);
    on<ChatUpdateListConversationEvent>(_onUpdateConversation);
    on<LoadUserInfoConversationEvent>(_onLoadUserInfo);
  }

  _onUpdateConversation(
      ChatUpdateListConversationEvent event, Emitter<ChatListState> emit) {
    emit(state.update(conversations: event.conversations));
  }

  onRemoveEvent(ChatListRemoveEvent event, Emitter<ChatListState> emit) {
    ChatHelper.removeConversation(event.conversationId);
    state.conversations
        .removeWhere((e) => e.conversationId == event.conversationId);
    emit(state.update());
  }

  _onReadItemEvent(ChatListReadItemEvent event, Emitter<ChatListState> emit) {
    if (event.c != null) {
      if (event.c!.isBot) {
        Get.toNamed(
          ChatDetailScreen.route,
          arguments: ChatDetailParams(
            event.c!.conversationType,
            isBot: true,
          ),
        );
      } else {
        if (event.c!.conversationType == ChatConversationType.solo) {
          String username = event.c!.userIds.first[kDbUSERNAME];
          Map? user = state.usersByKeys[username];
          String fullname = chatGetFullNameUser(user);
          String thumbnail = chatGetThumbnailUser(user);
          Get.toNamed(ChatDetailScreen.route,
              arguments: ChatDetailParams(
                event.c!.conversationType,
                peerFirebaseId: event.firebaseId,
                peerUsername: username,
                name: fullname,
                thumbnail: thumbnail,
              ));
        } else {
          Get.toNamed(
            ChatDetailScreen.route,
            arguments: ChatDetailParams(
              event.c!.conversationType,
              conversationId: event.c!.conversationId,
              name: event.c!.data[kDbGROUPNAME],
              thumbnail: event.c!.data[kDbTHUMBNAIL],
            ),
          );
        }
      }
    } else if (event.username != null) {
      Get.toNamed(
        ChatDetailScreen.route,
        arguments: ChatDetailParams(
          ChatConversationType.solo,
          peerFirebaseId: event.firebaseId,
          peerUsername: event.username,
          name: event.displayName,
          thumbnail: event.thumbnail,
        ),
      );
    }
  }

  _onDisposeEvent(event, Emitter<ChatListState> emit) {
    _converstationsSubscription?.cancel();
    for (var e in _msgsSubscription) {
      e.cancel();
    }
    _msgsSubscription.clear();
  }

  _onLoadUserInfo(
      LoadUserInfoConversationEvent event, Emitter<ChatListState> emit) async {
    if (!isLogged) return;
    var usersByKeys = state.usersByKeys;

    for (var username in event.usernames) {
      if (username != null) {
        Map<String, dynamic>? data =
            (await colUSERS.doc(username).get()).data();
        usersByKeys[username] = data;
      }
    }

    emit(state.update(
      usersByKeys: usersByKeys,
    ));
  }

  _onInitial(ChatListInitEvent event, Emitter<ChatListState> emit) async {
    if (!isLogged) return;
    emit(state.update(isLoading: true));
    QuerySnapshot q = await colCONVERSATIONS
        .where(kDbMEMBERS, arrayContains: loggedUsername)
        .orderBy(kDbLASTMESSAGETIME, descending: true)
        .get();
    List requestUsers = [];
    List<ChatListConversation> cons = [];
    if (chatOptions.botEnable) {
      cons.add(ChatListConversation(isBot: true));
    }
    for (var e in _msgsSubscription) {
      e.cancel();
    }
    _msgsSubscription.clear();
    for (var d in q.docs) {
      Map docData = d.data() as Map;
      int timestamp = docData[kDbMEMBERSDETAIL].firstWhere(
              (e) => e[kDbUSERNAME] == loggedUsername)[kDbTIMESTAMP] ??
          0;
      ChatListConversation conversation = ChatListConversation()
        ..conversationId = d.id
        ..userIds = docData[kDbMEMBERSDETAIL]
            .where((e) => e[kDbUSERNAME] != loggedUsername)
            .toList()
        ..conversationType =
            ChatConversationType.values.byName(docData[kDbCONVERSATIONTYPE])
        ..data = docData;
      requestUsers.addAll(docData[kDbMEMBERSDETAIL].toList());

      if (docData[kDbLASTMESSAGETIME] != null &&
          docData[kDbLASTMESSAGETIME] != 0) {
        conversation.timestamp = docData[kDbLASTMESSAGETIME];
        QuerySnapshot query = await colCONVERSATIONS
            .doc(d.id)
            .collection(kCollectionMESSAGES)
            .where(kDbTIMESTAMP, isGreaterThan: timestamp)
            .get();
        conversation.unread = query.size;

        //Last message
        DocumentSnapshot docLastMessage = await colCONVERSATIONS
            .doc(d.id)
            .collection(kCollectionMESSAGES)
            .doc('${docData[kDbLASTMESSAGETIME]}')
            .get();
        Map<String, dynamic> dataC = {};
        if (docLastMessage.data() != null) {
          dataC = docLastMessage.data() as Map<String, dynamic>;
          dataC[kDbCONTENT] = ChatHelper.decrypt(
              conversation.data[kDbENCRYPTKEY], dataC[kDbCONTENT]);
        }
        conversation.lastMessage = dataC;

        cons.add(conversation);
      } else {
        conversation.unread = 0;
        conversation.isEmptyMessages = true;
        conversation.lastMessage = {};
        cons.add(conversation);
      }
    }

    Map usersByKeys = await _getUsersByKeys(requestUsers);
    _getConnectionStatusUsers(requestUsers);

    emit(state.update(
      isLoading: false,
      conversations: cons,
      usersByKeys: usersByKeys,
    ));

    _converstationsSubscription ??= colCONVERSATIONS
        .where(kDbMEMBERS, arrayContains: loggedUsername)
        .snapshots()
        .listen((docs) {
      if (docs.docs.length != state.conversations.length) {
        add(ChatListInitEvent());
      }
    });

    for (var d in q.docs) {
      _msgsSubscription
          .add(colCONVERSATIONS.doc(d.id).snapshots().listen((snapshot) async {
        Map docData = snapshot.data() as Map;
        int timestamp = docData[kDbMEMBERSDETAIL].firstWhere(
                (e) => e[kDbUSERNAME] == loggedUsername)[kDbTIMESTAMP] ??
            0;
        ChatListConversation conversation = ChatListConversation()
          ..conversationId = d.id
          ..userIds = docData[kDbMEMBERSDETAIL]
              .where((e) => e[kDbUSERNAME] != loggedUsername)
              .toList()
          ..conversationType =
              ChatConversationType.values.byName(docData[kDbCONVERSATIONTYPE])
          ..data = docData;

        if (docData[kDbLASTMESSAGETIME] != null &&
            docData[kDbLASTMESSAGETIME] != 0) {
          conversation.timestamp = docData[kDbLASTMESSAGETIME];
          QuerySnapshot query = await colCONVERSATIONS
              .doc(d.id)
              .collection(kCollectionMESSAGES)
              .where(kDbTIMESTAMP, isGreaterThan: timestamp)
              .get();
          conversation.unread = query.size;

          //Last message
          DocumentSnapshot docLastMessage = await colCONVERSATIONS
              .doc(d.id)
              .collection(kCollectionMESSAGES)
              .doc('${docData[kDbLASTMESSAGETIME]}')
              .get();

          Map<String, dynamic> dataC = {};
          if (docLastMessage.data() != null) {
            dataC = docLastMessage.data() as Map<String, dynamic>;
            dataC[kDbCONTENT] = ChatHelper.decrypt(
                conversation.data[kDbENCRYPTKEY], dataC[kDbCONTENT]);
          }
          conversation.lastMessage = dataC;

          cons.add(conversation);
        } else {
          conversation.unread = 0;
          conversation.isEmptyMessages = true;
          conversation.lastMessage = {};
          cons.add(conversation);
        }

        List<ChatListConversation> conversations =
            List<ChatListConversation>.from(state.conversations
                    .where((e) => e.conversationId != d.id)) +
                [conversation];
        conversations.sort((a, b) => b.timestamp > a.timestamp ? 1 : -1);
        add(ChatUpdateListConversationEvent(conversations));
      }));
    }
  }

  void _getConnectionStatusUsers(List requestUsers) async {
    List<String> users =
        requestUsers.map((e) => e[kDbUSERNAME].toString()).toList();
    Get.find<ConnectionStatusBloc>().add(ConnectionStatusInitEvent(users));
  }

  Future<Map> _getUsersByKeys(requestUsers) async {
    Map usersByKeys = state.usersByKeys;
    for (var e in requestUsers) {
      if (state.usersByKeys[e[kDbUSERNAME]] == null) {
        Map<String, dynamic>? data =
            (await colUSERS.doc(e[kDbUSERNAME]).get()).data();
        usersByKeys[e[kDbUSERNAME]] = data;
      }
    }
    return usersByKeys;
  }
}
